#pragma once

// C++ system level
#include <string>
#include <functional>

// ESP32 specific
#include "freertos/FreeRTOS.h"
#include "freertos/queue.h"

// project specific
#include <SpiderLib.hpp>

// qthing stuff
// #include ""



namespace SiliconTorch {

  namespace CyanBus {

    // CyanBus payload MTU
    // This leads to ~54 P/s @ 2 MBaud at 100% load
    constexpr u32 MTU = 4096;

    // CyanBus envelope header
    extern const char* const HEADER;

    constexpr u8 UART_CHANNEL_AUTO = 0xFF;


    typedef struct {
      const u8* payload;
      const u16 length;
    } PacketData;

    typedef struct {
      const bool crcOK;
      const u16 payloadLEN;
    } PacketMetrics;


    class CyanBus;

    u32 createCyanBusInterface(u8 tx, u8 rx, u8 de, u8 re, u32 baudRate = 115200);
    u32 registerCyanBusInterface(CyanBus* iface);
    u32 countCyanBusInterfaces();
    CyanBus* getInterface(u32 number = 0);


    class CyanBus {
      public:
        CyanBus(u8 tx, u8 rx, u8 de, u8 re, u32 baudRate = 115200, u8 uartChannel = UART_CHANNEL_AUTO);

        u32 getBaudRate();
        void setBaudRate(u32 baudRate);

        const SpiderLib::CallbackMap<void(const PacketData&)> packetCallback;
        const SpiderLib::Callback<void(const PacketMetrics&)> metricsCallback;

      private:

        QueueHandle_t uartEvQ;
        QueueHandle_t packetQ;

        u8 uartChannel;
        u8 tx, rx, de, re;

        void txEN(bool state);
        void rxEN(bool state);
        void txrxEN(bool state);


        // Receive only ATM!  (make public after sound implementation…)
        void write(std::string& data);
        void write(const u8* data, u32 length);


        void readerTask();
        void packetTask();

        SpiderLib::Util::LambdaTask* readerTaskHandle = NULL;
        SpiderLib::Util::LambdaTask* packetTaskHandle = NULL;

    };
  }
}
