
import shutil
import subprocess


class SCADFile:
  def __init__(self, filename):
    self.__filename = filename
    self.content = '\n'
    self.indent = 0

  def print(self, any, newLines=1):

    lineStart = ' ' * self.indent
    lineEnd   = '\n' * newLines

    self.write(f'{lineStart}{any}{lineEnd}')

  def write(self, any):
    self.content += str(any)

  def save(self):
    with open(self.__filename, 'w') as f:
      f.write(self.content)

  def __enter__(self):
    return self

  def __exit__(self, *ignored):
    # Yes we don't behave like a real file object here as we may want to build an API to change the content before write-out
    # which is obviously impossible if we write the file line-by-line like a real file object
    self.save()


class OpenSCAD:

  def __init__(self):
    self.__openscad = shutil.which('openscad')

  def __call__(self, *args, outfile=None, csglimit=None, **kwargs):
    '''args are added at the end, kwargs are used to od variable overrides'''

    cmd = [self.__openscad]


    # Can we do better instead of replicating this all over the place???

    if csglimit is not None:
      cmd += [f'--csglimit={csglimit}']


    for k, v in kwargs.items():
      cmd += ['-D', f'{k}={v}']


    if outfile is not None:
      cmd += ['-o', outfile]

    cmd += args

    print(f'Running OpenSCAD: {cmd}')  # TODO: remove after debugging?!
    return subprocess.Popen(cmd).communicate()
    


